function cost = Cost(Ahat, xbar, beta, x, u, lambda, T)
% actual realized cost
%
% Input:
%
% Ahat: (p+1)-by-(p+1) matrix, the forecaster's AR(p) model
%
% xbar: (T+1)-by-(T+1) matrix, scalar version of adversarial target.
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. For xbar(t+1,t'+1) 
% that is: t < 0 or t>=t' or t=T, xbar(t+1,t'+1) should be set to 0.
%
% beta: (T+1)-by-(T+1) matrix, nonzero entry of Q matrix, beta(t+1,t'+1)
% corresponds to the nonzero entry in the quadratic cost matrix Q_{t'|t} in 
% paper. For beta(t+1,t'+1) that is: t<0 or t>=t' or t=T, beta(t+1,t'+1)
% should be set to 0.
%
% x: (p+1)-by-(T+1) matrix, real state value sequence under control u: \bfx_0, ..., \bfx_{T}
%
% u: (1)-by-(T) matrix, attack action sequence: u_0, ..., u_{T-1}
%
% lambda>0: adversarial effort parameter
%
% T: prediction time horizon
%
%
% Output:
% realized cost


% retrieve p
p = length(Ahat(:,1)) - 1;

cost = 0;

for t = 1 : T-1
	for tp = t+1 : T
		xbarbf = zeros(p+1, 1);
		xbarbf(2,1) = xbar(t+1, tp+1);
		Q = zeros(p+1, p+1);
		Q(2,2) = beta(t+1, tp+1);
		cost = cost + (Ahat^(tp - t) * x(:,t+1) - xbarbf)' * Q * (Ahat^(tp - t) * x(:,t+1) - xbarbf);
	end
end

usum = 0;
for t = 0 : T-1
	usum = usum + u(t+1)^2;
end
cost = cost + lambda * usum;

end