% this code is used to generate plot 'figexper2_lambdas.eps' for the toy example in section 4.2.
%
% A, B is the environment dynamic;
% Ahat is forecast matrix
% x0 is the scalar initial state
% T is the time horizon
% bar is the attack reference target (value)
% lambdatilde is \tilde\lambda in paper

clear;

A = [1, 0; 1, 0.5];
Ahat = [1, 0; 0.9, 0.6];
B = [0; 1];
x0 = 0;
T = 10;

sim = 1;

balance1 = 10^(-3);
balance2 = 10^(1);
bar = 1; % xbar is set to be 1 when the prediction is considered by attacker

% set lambda to balance the different sum number
lambda = @(beta, balance) balance*sum(sum(beta))/T;

F = @(x, u, w, mode) A * x + B * u + B * w;

% when attacak target is to change tomorrow's prediction
beta_tom = diag(ones(1,T),1);
xbar_tom = bar * beta_tom;

% used fixed random seed to generate noise sequence
rng(1);

for iSim = 1:sim
	w = normrnd(0, 0.1, [1, T]);	
	
	% when there is no attack, compute prediction, state value, control sequence (zeros)
	[xhat_non, x_non, u_non] = Pred(F, Ahat, x0, T, zeros(1, T), w);	

	% ----------------------------------------------------------------------------------
	% when attacak target is to change tomorrow's prediction	

	% compute policy functions of LQR anf Greedy
	PHI_lqr_tom_1 = LQRStateAttack(A,Ahat,xbar_tom,beta_tom,lambda(beta_tom, balance1),T);
	PHI_lqr_tom_2 = LQRStateAttack(A,Ahat,xbar_tom,beta_tom,lambda(beta_tom, balance2),T);

	% compute state, attack sequence and prediction sequence uder LQR and Greedy policy
	[xhat_lqr_tom_1, x_lqr_tom_1, u_lqr_tom_1] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_lqr_tom_1(t,x), w);
	[xhat_lqr_tom_2, x_lqr_tom_2, u_lqr_tom_2] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_lqr_tom_2(t,x), w);
	

end

figtoy1_5 = figure

% when there is no attacker
ax1 = subplot(1,3,1);
for t = 0 : T-1
	p1 = plot([t+1:T], xhat_non(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x_non(2,:), '-m.');
axis('square');
xlabel('time');
title(['no attack']);
legend([p1, p2], {'forecast $\hat x_{t'' \mid t} $ ', 'env state $x_t$'}, 'Location', 'southeast','interpreter','latex');

% when attacak target is to change tomorrow's prediction
% lambda ~ 0.001
ax2 = subplot(1,3,2);
[p1, p2, p3] = Predplot(xhat_lqr_tom_1, x_lqr_tom_1, u_lqr_tom_1, T);
p4 = plot([1:T], diag(xbar_tom, 1), '-g.');
p5 = plot([1:T], diag(xhat_lqr_tom_1, 1), '*b');
axis('square');
legend([p3, p4], {'LQR $u_t$', 'target $\bar x_{t+1 \mid t}$'}, 'Location', 'northwest','interpreter','latex');
str0 = ['attack predictions for '];
str1 = ['tomorrows: $\hat x_{t+1 \mid t}$, $\tilde\lambda = 0.001$'];
str = {str0;str1};
title(str,'interpreter','latex');

% when attacak target is to change tomorrow's prediction
% lambda ~ 1000
ax3 = subplot(1,3,3);
[p1, p2, p3] = Predplot(xhat_lqr_tom_2, x_lqr_tom_2, u_lqr_tom_2, T);
p4 = plot([1:T], diag(xbar_tom, 1), '-g.');
p5 = plot([1:T], diag(xhat_lqr_tom_2, 1), '*b');
axis('square');
str0 = ['attack predictions for '];
str1 = ['tomorrows: $\hat x_{t+1 \mid t}$, $\tilde\lambda = 1$'];
str = {str0;str1};
title(str,'interpreter','latex');

linkaxes([ax1,ax2,ax3],'xy')

saveas(figtoy1_5,'figexper2_lambdas','epsc');


function [p1, p2, p3] = Predplot(xhat, x, u, T)
for t = 0 : T-1
	p1 = plot([t+1:T], xhat(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x(2,:), '-m.');
p3 = plot([0:T-1], u, '-r.');

xlabel('time');

end
